#include <LiquidCrystal.h>
#include <Wire.h>
#include <Ultrasonic.h>

Ultrasonic sensor_B(3, 2);
Ultrasonic sensor_F(4, 5);
LiquidCrystal lcd(13, 12, 11, 10, 9, 8);

#define IMPACT_DIST 4
#define BEGIN_DIST 34
#define SOUND_SPEED 0.03466
#define BUZZER 6

bool hit = 0, safe_range = 1;
int pause_time, break_millis;
String readString;

void setup() {
  Serial.begin(9600);
  lcd.begin(16, 2);
  pinMode(6, OUTPUT);
  DDRD |= (1 << PD7); // led

  // back
  DDRD |= (1 << PD3);
  DDRD &= ~(1 << PD2);

  // front
  DDRD |= (1 << PD4);
  DDRD &= ~(1 << PD5);
}

int get_duration(bool front) {
  int trig = front == true ? 4 : 3, echo = front == true ? 5 : 2;
  unsigned long previousMicros;

  PORTD &= ~(1 << trig);
  delayMicroseconds(2);
  PORTD |= (1 << trig);
  delayMicroseconds(10);
  PORTD &= ~(1 << trig);

  previousMicros = micros();
  while(!digitalRead(echo) && (micros() - previousMicros) <= 20000UL);
  previousMicros = micros();
  while(digitalRead(echo)  && (micros() - previousMicros) <= 20000UL);

  return micros() - previousMicros;
}
  
void print_dist_serial(float back, float front) {
  Serial.println("================");
  Serial.print("Back: ");
  Serial.println(back);
  
  Serial.print("Front: ");
  Serial.println(front);
}

void print_dist_lcd(float back, float front) {
  if (safe_range == 0)
    delay(100);
  
  lcd.clear();
  lcd.print("Back: ");
  if (back > BEGIN_DIST)
    lcd.print("N/A");
  else if (back < IMPACT_DIST)
    lcd.print("HIT");
  else
    lcd.print(back);
  
  lcd.setCursor(0, 1); // set the cursor to column 0, line 2
 
  lcd.print("Front: ");
  if (front > BEGIN_DIST)
    lcd.print("N/A");
  else if (front < IMPACT_DIST)
    lcd.print("HIT");
  else
    lcd.print(front);

  lcd.setCursor(0, 0);// set the cursor to column 0, line1
}

bool buzz(float left, float right) {
  float min_val = min(left, right);
  if (min_val > BEGIN_DIST) {
    safe_range = 1;
    return false;
  }
  int pause_time = 10.5 * pow(2, 1.5 * min_val / 10) - 10.5;
  
  analogWrite(BUZZER, 127);
  if (min_val > 7) {
    safe_range = 1;
    delay(100);
    analogWrite(BUZZER, 0);
    delay(pause_time);
  } else {
    safe_range = 0;
    break_millis = millis();
    delay(5);
  }
  return true;
}

char rxbuf[20];
int index = 0;
long ts = 0;

void loop() {
  long microsec_B = get_duration(0), microsec_F = get_duration(1);
  float left_dist = microsec_B * SOUND_SPEED / 2;
  float right_dist = microsec_F * SOUND_SPEED / 2;
  
  print_dist_serial(left_dist, right_dist);
  
  if ((left_dist < IMPACT_DIST || right_dist < IMPACT_DIST) && hit == 0) {
    PORTD |= (1 << PD7);
    hit = 1;
  }

  if (buzz(left_dist, right_dist) == false)
    delay(300);

  print_dist_lcd(left_dist, right_dist);

  if (Serial.available()) {
      char a = Serial.read();
      if (a == '\n') {
        rxbuf[index] = '\0';
        if (strcmp(rxbuf, "reset") == 0) {
          PORTD &= ~(1 << PD7);
          hit = 0;
        }
        index = 0;
        rxbuf[index] = '\0';
      } else {
        rxbuf[index] = a;
        index++;
        rxbuf[index] = '\0';
      }
   }


}
